"""
IAMAuth using Signatuve v4a signing process.
"""

from io import BytesIO

from botocore.awsrequest import AWSRequest
from botocore.compat import awscrt, parse_qsl, urlsplit
from requests.auth import AuthBase


class Sigv4aAuth(AuthBase):
    """
    AWS IAM authorizer for sigv4a.

    This implementation owes a lot to the blog post here:
    https://chammock.dev/posts/aws-apigw-multi-region-iam-auth/

    :param boto3.Session session: Optional boto3 Session object
    :param str service_name: Optional AWS service name

    :Example:

    >>> IAMAuth()
    >>> IAMAuth("execute-api")
    >>> IAMAuth("execute-api", "us-*")
    >>> IAMAuth("execute-api", "us-*", my_credentials_provider)
    """

    def __init__(self, service=None, region=None, credentials_provider=None):
        self.service = service or "execute-api"
        self.region = region or "*"
        self.credentials_provider = (
            credentials_provider
            or awscrt.auth.AwsCredentialsProvider.new_default_chain()
        )

    def __call__(self, request):
        # Parse request URL
        url = urlsplit(request.url)

        # Prepare AWS request
        aws_url = f"{url.scheme}://{url.netloc}{url.path}"
        aws_headers = dict(request.headers)
        aws_params = parse_qsl(url.query)
        aws_request = AWSRequest(
            method=request.method,
            url=aws_url,
            headers=aws_headers,
            data=request.body,
            params=aws_params,
        )

        # Prepare CRT request
        crt_host = request.headers.get("host", url.netloc)
        crt_headers = awscrt.http.HttpHeaders([("host", crt_host)])
        crt_path = url.path or "/"
        crt_body_stream = BytesIO(request.body) if request.body else None
        crt_request = awscrt.http.HttpRequest(
            method=request.method,
            path=crt_path,
            headers=crt_headers,
            body_stream=crt_body_stream,
        )

        # Setup signing config
        signing_config = awscrt.auth.AwsSigningConfig(
            algorithm=awscrt.auth.AwsSigningAlgorithm.V4_ASYMMETRIC,
            signature_type=awscrt.auth.AwsSignatureType.HTTP_REQUEST_HEADERS,
            credentials_provider=self.credentials_provider,
            region=self.region,
            service=self.service,
        )

        # Sign CRT request and merge headers into AWS request
        awscrt.auth.aws_sign_request(crt_request, signing_config).result()
        signed_headers = dict(crt_request.headers)
        aws_request.headers = {**aws_headers, **signed_headers}

        # Return prepared AWS request
        return aws_request.prepare()



from urllib.parse import urlparse, parse_qsl

import requests
from botocore.compat import awscrt

from iamauth import Sigv4aAuth


class TestIAMAuth:
    def setup_method(self):
        credentials_provider = awscrt.auth.AwsCredentialsProvider.new_static(
            access_key_id="<access-key>",
            secret_access_key="<secret-key>",
            session_token="<session-token>",
        )
        self.subject = Sigv4aAuth(credentials_provider=credentials_provider)

    def test_call(self):
        req = requests.Request(
            "GET",
            "http://example.com/",
            auth=self.subject,
            headers={"Content-Type": "application/json; charset=utf-8"},
        )
        prq = req.prepare()
        assert "X-Amz-Date" in prq.headers
        assert "Authorization" in prq.headers

    def test_call_with_query_params(self):
        req = requests.Request(
            "GET",
            "http://example.com/",
            auth=self.subject,
            params={"singleValueParam": "val1", "multiValueParam": ["val2", "val3"]},
        )
        prq = req.prepare()
        query = urlparse(prq.url).query
        assert parse_qsl(query) == [
            ("singleValueParam", "val1"),
            ("multiValueParam", "val2"),
            ("multiValueParam", "val3"),
        ]



import boto3
from botocore.auth import SigV4Auth
from botocore.awsrequest import AWSRequest
from botocore.compat import parse_qsl, urlsplit
from requests.auth import AuthBase


class Sigv4Auth(AuthBase):
    """
    AWS IAM authorizer for sigv4.

    :param boto3.Session session: Optional boto3 Session object
    :param str service_name: Optional AWS service name

    :Example:

    >>> IAMAuth()
    >>> IAMAuth("execute-api")
    >>> IAMAuth("execute-api", boto3.Session())
    """

    def __init__(self, service_name="execute-api", boto3_session=None):
        self.boto3_session = boto3_session or boto3.Session()
        self.sigv4 = SigV4Auth(
            credentials=self.boto3_session.get_credentials(),
            service_name=service_name,
            region_name=self.boto3_session.region_name,
        )

    def __call__(self, request):
        # Parse request URL
        url = urlsplit(request.url)

        # Prepare AWS request
        awsrequest = AWSRequest(
            method=request.method,
            url=f"{url.scheme}://{url.netloc}{url.path}",
            data=request.body,
            params=parse_qsl(url.query),
        )

        # Sign request
        self.sigv4.add_auth(awsrequest)

        # Re-add original headers
        for key, val in request.headers.items():
            if key not in awsrequest.headers:
                awsrequest.headers[key] = val

        # Return prepared request
        return awsrequest.prepare()



from types import SimpleNamespace
from unittest.mock import MagicMock

import pytest

from iamauth import __main__ as index


class TestIAMCURL:
    def setup_method(self):
        index.get_args = MagicMock()
        index.requests.Session = MagicMock()

    @pytest.mark.parametrize("req,url", [("GET", "https://example.com/path")])
    def test_main(self, req, url):
        index.get_args.return_value = SimpleNamespace(
            data=None,
            header=[],
            request=req,
            sigv4a=False,
            URL=url,
        )
        index.main()
        index.requests.Session.return_value.request.assert_called_once_with(
            "GET",
            "https://example.com/path",
            headers={},
            data=None,
        )



import argparse

import requests

from . import __version__, Sigv4Auth, Sigv4aAuth


def get_args():
    parser = argparse.ArgumentParser(
        description="IAM-authorized HTTP request helper",
        prog="iamcurl",
        usage="%(prog)s [OPTIONS] URL",
    )
    parser.add_argument("-v", "--version", action="version", version=__version__)
    parser.add_argument("-d", "--data", help="Request data")
    parser.add_argument("-H", "--header", action="append", help="Request header")
    parser.add_argument("-X", "--request", help="HTTP request method")
    parser.add_argument("--sigv4", action="store_true", help="Use sigv4 algorithm")
    parser.add_argument("--sigv4a", action="store_true", help="Use sigv4a algorithm")
    parser.add_argument("URL", help="URL to request")
    args = parser.parse_args()
    return args


def main():
    args = get_args()
    IAMAuth = Sigv4aAuth if args.sigv4a else Sigv4Auth
    session = requests.Session()
    session.auth = IAMAuth()
    method = args.request or "GET"
    url = args.URL
    headers = dict(x.split(": ") for x in args.header) if args.header else {}
    result = session.request(method, url, headers=headers, data=args.data)
    return result.text


if __name__ == "__main__":  # pragma: no cover
    print(main())



from urllib.parse import urlparse, parse_qsl

import boto3
import requests

from iamauth import Sigv4Auth


class TestSigv4Auth:
    def setup_method(self):
        self.boto3_session = boto3.Session(
            aws_access_key_id="<access>",
            aws_secret_access_key="<secret>",
            region_name="us-east-2",
        )
        self.subject = Sigv4Auth(boto3_session=self.boto3_session)

    def test_call(self):
        req = requests.Request(
            "GET",
            "http://example.com/",
            auth=self.subject,
            headers={"Content-Type": "application/json; charset=utf-8"},
        )
        prq = req.prepare()
        assert "X-Amz-Date" in prq.headers
        assert "Authorization" in prq.headers

    def test_call_with_query_params(self):
        req = requests.Request(
            "GET",
            "http://example.com/",
            auth=self.subject,
            params={"singleValueParam": "val1", "multiValueParam": ["val2", "val3"]},
        )
        prq = req.prepare()
        query = urlparse(prq.url).query
        assert parse_qsl(query) == [
            ("singleValueParam", "val1"),
            ("multiValueParam", "val2"),
            ("multiValueParam", "val3"),
        ]
